"""
A python library for working with the speedrun.com API.
This library is intended to make the api easier to use,
by preprocessing the data, and separating it into specific sub categories.
"""

from .speedrunapi import *
from speedrunapi.game_data import *
from speedrunapi.user_data import *
from speedrunapi.runs import *
from speedrunapi.levels import *
from speedrunapi.regions import * 
from speedrunapi.leaderboard import *
from speedrunapi.categories import *
